/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "DiscoverClient.h"

#include "shared/DiscoverProtocol.h"
#include "shared/defaultValues.h"

// Qt includes
#include <QDebug>
#include <QTimer>

using namespace DiscoverProtocol;

DiscoverClient::DiscoverClient(QObject *parent) : AbstractDiscoverEngine(parent)
{
    m_timer = new QTimer(this);
    m_timer->setInterval(NETWORK_DISCOVER_TIMEOUT);
    m_timer->setSingleShot(true);
    connect(m_timer, &QTimer::timeout, this, &DiscoverClient::discoverTimeout);
}

bool DiscoverClient::discoverServer()
{
    if (! listen()) {
        return false;
    }

    m_timer->start();
    broadcastData(MESSAGE_SEARCHING_SERVER);
    return true;
}

void DiscoverClient::discoverTimeout()
{
    close();
    emit discoverResult(QString(), -1);
}

void DiscoverClient::processData(const QJsonObject &jsonObject, const QHostAddress &)
{
    if (jsonObject.value(KEY_ACTION) != VALUE_ANNOUNCING_SERVER) {
        return;
    }

    m_timer->stop();
    close();
    emit discoverResult(jsonObject.value(KEY_IP).toString(), jsonObject.value(KEY_PORT).toInt());
}
