/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef IMPORTPLAYERSDIALOG_H
#define IMPORTPLAYERSDIALOG_H

// Local includes
#include "shared/FileDialog.h"

// Local classes
class SharedObjects;
class StringTools;
class SearchEngine;

// Qt classes
class QLabel;
class QTableWidget;
class QGroupBox;
class QCheckBox;
class QTableWidgetItem;

class ImportPlayersDialog : public FileDialog
{
    Q_OBJECT

public:
    explicit ImportPlayersDialog(QWidget *parent, SharedObjects *sharedObjects);

signals:
    void updateStatus(const QString &text);

private slots:
    void prepareImport();
    void updateImportCount(QTableWidgetItem *);
    virtual void accept() override;

private: // Functions
    void importFile();
    void findPossibleDuplicates();
    void displayUnambiguousNames();
    void resizeView();

private slots:
    void showDuplicatesSearchProgress(int percent);

private: // Variables
    StringTools *m_stringTools;
    SearchEngine *m_searchEngine;
    QString m_importPlayersListString;
    QString m_title;
    QLabel *m_unambiguousNamesLabel;
    QTableWidget *m_unambiguousNames;
    QGroupBox *m_possibleDuplicatesBox;
    QLabel *m_possibleDuplicatesLabel;
    QTableWidget *m_possibleDuplicates;
    QStringList m_unambiguousNamesList;
    QGroupBox *m_duplicatesBox;
    QCheckBox *m_addDuplicatesPrefix;
    QString m_duplicatesPrefix;

};

#endif // IMPORTPLAYERSDIALOG_H
