/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MARKERSWIDGET_H
#define MARKERSWIDGET_H

// Local includes
#include "shared/Database.h"
#include "shared/Marker.h"

// Qt includes
#include <QWidget>
#include <QIcon>
#include <QCryptographicHash>

// Local classes
class SharedObjects;
class TitleMenu;
class Server;
class Client;

// Qt classes
class QTableWidget;
class QAction;
class QMenu;
class QComboBox;
class QLineEdit;
class QGroupBox;

class MarkersWidget : public QWidget
{
    Q_OBJECT

public:
    enum MarkerData {
        Id = Qt::UserRole,
        Sorting
    };

    explicit MarkersWidget(QWidget *parent, SharedObjects *sharedObjects);
    void reload();
    QList<Marker::Marker> getMarkers() const;
    void reset();
    bool isMarkerVisible(const int id) const;
    void updateCounters(const QMap<int, int> &counter);
    int defaultMarker(const QString &name) const;
    void addChecksumData(QCryptographicHash &hash) const;
    void setServer(Server *server);
    void setClient(Client *client);
    void serverStopped();
    void clientDisconnected();
    void processNetworkChange();
    void setProcessingRequest(const bool processing);
    void setAutomaticMarkingBoxVisible(const bool state);

signals:
    void markersChanged();
    void statusUpdate(const QString &text);
    void visibilityChanged();

private slots:
    void showContextMenu(const QPoint &position);
    void addMarker();
    void saveNewMarker(const QString name, const QColor color,
                       const Marker::Sorting sorting);
    void moveUp();
    void moveDown();
    void editMarker();
    void saveEdit(const QString name, const QColor color, const Marker::Sorting sorting);
    void deleteMarker();
    void checkShowContextMenu(int, const int column);
    void changeVisibility(const int row, const int column);
    void showAll();
    void showOnly(const int idToShow);

private: // Functions
    void updateVisibilityIcons();
    void updateVisibilityIcon(const int id, const int row);
    bool checkSelectedMarkerChanged(const QString &name, const QString &action);

private: // Variables
    Database *m_db;
    QTableWidget *m_markers;
    TitleMenu *m_markerMenu;
    QAction *m_editMarkerAction;
    QAction *m_moveUpAction;
    QAction *m_moveDownAction;
    QAction *m_deleteMarkerAction;
    QAction *m_addMarkerAction;
    QMenu *m_addMarkerMenu;
    QMenu *m_hideShowMarkedMenu;
    QAction *m_showOnlyAction;
    QAction *m_showAllAction;
    Marker::Marker m_selectedMarker;
    int m_selectedMarkerSequence;
    QIcon m_unused;
    QIcon m_visible;
    QIcon m_hidden;
    QMap<int, bool> m_isVisible;
    int m_allMarkersCount;
    QGroupBox *m_automaticMarkingBox;
    QComboBox *m_defaultMarker;
    QGroupBox *m_conditionalMarkerBox;
    QComboBox *m_markerCondition;
    QLineEdit *m_markerConditionString;
    QComboBox *m_conditionMarker;

    Server *m_server;
    Client *m_client;
    bool m_isServer = false;
    bool m_isClient = false;
};

#endif // MARKERSWIDGET_H
