/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "PlayersPageDelegate.h"
#include "PlayersEditor.h"

// Qt includes
#include <QLineEdit>
#include <QDebug>

PlayersPageDelegate::PlayersPageDelegate(QObject *parent) : TableDelegate(parent)
{
    connect(this, &QAbstractItemDelegate::closeEditor, [this] { m_isEditing = false; } );
}

QString PlayersPageDelegate::editedText() const
{
    return m_isEditing ? m_originalData : QString();
}

QString PlayersPageDelegate::enteredText() const
{
    return m_isEditing ? m_editor->text() : QString();
}

bool PlayersPageDelegate::isEditing() const
{
    return m_isEditing;
}

void PlayersPageDelegate::saveEditorState()
{
    m_enteredText = m_editor->text();
    if (m_editor->hasSelectedText()) {
        if (m_editor->cursorPosition() == m_editor->selectionStart()) {
            m_selectionStart = m_editor->selectionStart() + m_editor->selectedText().length();
            m_selectionLength = m_editor->selectedText().length() * -1;
        } else {
            m_selectionStart = m_editor->selectionStart();
            m_selectionLength = m_editor->selectedText().length();
        }
    } else {
        m_selectionStart = -1;
        m_cursorPosition = m_editor->cursorPosition();
    }
    abortEdit();
}

void PlayersPageDelegate::abortEdit()
{
    emit closeEditor(m_editor);
}

void PlayersPageDelegate::restoreEditorState()
{
    m_editor->setText(m_enteredText);
    if (m_selectionStart != -1) {
        m_editor->setSelection(m_selectionStart, m_selectionLength);
    } else {
        m_editor->setCursorPosition(m_cursorPosition);
    }
}

QWidget * PlayersPageDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &,
                                            const QModelIndex &index) const
{
    emit startedEdit(index.row());
    m_isEditing = true;
    m_originalData = index.data().toString();
    m_editor = new PlayersEditor(m_originalData, parent);
    return m_editor;
}

void PlayersPageDelegate::setModelData(QWidget *, QAbstractItemModel *, const QModelIndex &) const
{
    if (! m_editor->enterPressed()) {
        // The editing has been finished because the editor lost it's focus
        return;
    }

    const QString enteredData = enteredText().simplified();
    if (enteredData.isEmpty() || enteredData == m_originalData) {
        return;
    }
    emit editEntered(m_originalData, enteredData);
}
