/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RANKINGDIALOG_H
#define RANKINGDIALOG_H

// Local includes
#include "shared/TitleDialog.h"
#include "shared/Database.h"

// Local classes
class SharedObjects;
class StringTools;
class Client;

// Qt classes
class QLabel;
class QTableWidget;
class QPushButton;

class RankingDialog : public TitleDialog
{
    Q_OBJECT

public:
    explicit RankingDialog(QWidget *parent, SharedObjects *sharedObjects, Client *client);
    void noRanking();
    void compareRanking(QList<Database::RankData> &remoteRanking);

public slots:
    void clientDisconnected();
    void dataChanged();

private: // Functions
    void requestRanking();
    void dataReceived();
    const QString differenceString(int local, int remote) const;
    void resizeToTableWidth();
    void finishDisplayDifferences();
    bool checkTournamentStarted();

private: // Variables
    Database *m_db;
    Client *m_client;
    StringTools *m_stringTools;
    QLabel *m_dataChangedLabel;
    QLabel *m_resultLabel;
    QTableWidget *m_differences;
    QPushButton *m_requestRanking;
    QPushButton *m_close;

};

#endif // RANKINGDIALOG_H
