/* Copyright (c) 2012-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SCOREPAGE_H
#define SCOREPAGE_H

// Local includes
#include "shared/Database.h"

// Qt includes
#include <QWidget>
#include <QMap>

// Local classes
class SharedObjects;
class StringTools;
class SearchEngine;
class BoogerWidget;
class TitleMenu;
class NumbersComboBox;
class PlayersComboBox;
class BoogerSpinBox;
class ScorePopup;
class SearchWidget;
class Settings;

// Qt classes
class QGroupBox;
class QCheckBox;
class QScrollArea;
class QPushButton;
class QTableWidget;
class QAction;
class QLabel;

class ScorePage : public QWidget
{
    Q_OBJECT

public:
    explicit ScorePage(QWidget *parent, SharedObjects *sharedObjects);
    void setupGui();
    void reload(int lastRound = -1);
    void setScrollAreaHeight();
    void setNoTableNumbers(bool state);
    void setAutoSelectPairs(bool state);
    void resetSearch();
    void playersNameEdited();

signals:
    void statusUpdate(QString message);
    void tournamentStarted();
    void scoreChanged();

protected:
    virtual void showEvent(QShowEvent *event) override;

private slots:
    void openRound(const int round);
    void tableChanged(const int table);
    void saveScore();
    void showContextMenu(QPoint point);
    void editScore();
    void deleteScore();
    void checkEnterScore();
    void updateSearch(const QString &searchText);
    void blockSelectSignals(bool state);
    void swapPairs();
    void noTableNumbersChanged(bool state);
    void autoSelectPairsChanged(bool state);
    void boogerAbortedStateChanged(int booger, bool aborted);
    void disableSaveScore();
    void checkSavePossible();
    void showScorePopup(BoogerSpinBox *spinBox);
    void setSelectedScore();

private: // Functions
    void updateScore();
    void playersChanged(const int pair, const int pairId);
    void doDeleteScore(int round, int table);
    const QString getStatusUpdateIdentifier() const;
    const QString formatOpponentsString(const QString &pair1, const QString &pair2) const;
    void fillInScore(int round, int table, const QList<Database::BoogerResult> &score);
    const QList<Database::BoogerResult> getCurrentScore() const;

private: // Variables
    StringTools *m_stringTools;
    Database *m_db;
    SearchEngine *m_searchEngine;
    NumbersComboBox *m_roundSelect;
    QGroupBox *m_resultBox;
    NumbersComboBox *m_tableSelect;
    QMap<int, QMap<int, PlayersComboBox *>> m_playersSelect;
    QCheckBox *m_autoSelectPairs;
    QScrollArea *m_scoreBoxScroll;
    QPushButton *m_saveScoreButton;
    QGroupBox *m_roundSummaryBox;
    QTableWidget *m_scoreList;
    TitleMenu *m_contextMenu;
    QAction *m_editAction;
    QAction *m_deleteAction;
    QMap<int, bool> m_roundsState;
    Database::Assignment m_assignment;
    QList<BoogerWidget *> m_boogerWidgets;
    SearchWidget *m_searchWidget;
    QLabel *m_isSearchLabel;
    QAction *m_swapPairsAction;
    int m_lastOpenedRound;
    int m_selectedTable;
    QCheckBox *m_noTableNumbers;
    QLabel *m_tableLabel;
    bool m_lastAutoSelectState;
    QStringList m_availableNames;
    ScorePopup *m_scorePopup;
    BoogerSpinBox *m_targetSpinBox;
    Settings *m_settings;

};

#endif // SCOREPAGE_H
