/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "ScorePopup.h"

// Qt includes
#include <QDebug>
#include <QGridLayout>
#include <QPushButton>

// C++ includes
#include <functional>

ScorePopup::ScorePopup(QWidget *parent) : SelectionPopup(parent)
{
}

void ScorePopup::setBoogerScore(const int score)
{
    if (m_boogerScore == score) {
        return;
    }

    QWidget *scoreWidget = new QWidget;
    QGridLayout *layout = new QGridLayout(scoreWidget);

    int maximumColumns;
    if (score <= 31) {
        maximumColumns = 5;
    } else {
        maximumColumns = 10;
    }

    int row = -1;
    int currentScore = -1;
    bool finished = false;
    while (! finished) {
        row++;
        for (int column = 0; column < maximumColumns; column++) {
            currentScore++;
            if (currentScore == score) {
                finished = true;
                break;
            }

            QPushButton *scoreButton = new QPushButton(QString::number(currentScore));
            layout->addWidget(scoreButton, row, column);
            connect(scoreButton, &QPushButton::clicked,
                    std::bind(&ScorePopup::scoreClicked, this, scoreButton));
        }
    }

    setScrollWidget(scoreWidget);
    m_boogerScore = score;
}

void ScorePopup::scoreClicked(QPushButton *button)
{
    m_score = button->text().toInt();
    // This is needed to prevent the button to be highlighted afterwards on Windows
    button->clearFocus();
    accept();
}

int ScorePopup::score() const
{
    return m_score;
}
