/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONNECTION_H
#define CONNECTION_H

// Local includes
#include "shared/AbstractNetworkEngine.h"

// Qt includes
#include <QAbstractSocket>

// Qt classes
class QJsonObject;

class Connection : public AbstractNetworkEngine
{
    Q_OBJECT

public:
    explicit Connection(QObject *parent);
    bool setSocketDescriptor(qintptr descriptor);
    void close();
    void setId(int id);
    int id() const;

signals:
    void error(QAbstractSocket::SocketError error);
    void disconnected();
    void handshakeResult(const QString &phase);
    void clientRequest(const QString &action, const QJsonObject &jsonObject);

private: // Functions
    virtual void dataReceived(const QString &action, const QJsonObject &jsonObject) override;
    void processHandshake(const QJsonObject &jsonObject);

private: // Variables
    int m_id;

};

#endif // CONNECTION_H
