/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "DiscoverServer.h"

#include "shared/DiscoverProtocol.h"

// Qt includes
#include <QDebug>
#include <QJsonValue>

// C++ includes
#include <functional>

using namespace DiscoverProtocol;

DiscoverServer::DiscoverServer(QObject *parent) : AbstractDiscoverEngine(parent)
{
    connect(this, &AbstractDiscoverEngine::invalidData,
            this, std::bind(&DiscoverServer::invalidRequest, this, std::placeholders::_1,
                            tr("Ungültiger Datensatz")));
}

bool DiscoverServer::serve(const QString &ip, const int port)
{
    m_serverAnnouncement = QJsonObject { { KEY_ACTION, VALUE_ANNOUNCING_SERVER },
                                         { KEY_IP,     ip },
                                         { KEY_PORT,   port } };
    return listen();
}

void DiscoverServer::invalidRequest(const QHostAddress &senderAddress, const QString &text)
{
    emit message(tr("Ungültige Anfrage von %1: %2").arg(senderAddress.toString(), text));
}

void DiscoverServer::processData(const QJsonObject &jsonObject, const QHostAddress &senderAddress)
{
    if (jsonObject.value(KEY_ACTION).toString() != VALUE_SEARCHING_SERVER) {
        invalidRequest(senderAddress, tr("Ungültige Server-Suchanfrage"));
        return;
    }

    const int protocolVersion = jsonObject.value(KEY_PROTOCOL_VERSION).toInt();
    if (protocolVersion != PROTOCOL_VERSION) {
        invalidRequest(senderAddress, tr("Falsche Protokollversion (%1)").arg(protocolVersion));
        return;
    }

    emit message(tr("Server-Suchanfrage von %1 erhalten. Sende Socket-Adresse").arg(
                    senderAddress.toString()));
    sendData(m_serverAnnouncement, senderAddress);
}
