/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SERVER_H
#define SERVER_H

// Local includes
#include "shared/Database.h"
#include "shared/Marker.h"

// Qt includes
#include <QTcpServer>

// Local classes
class SharedObjects;
class PlayersPage;
class MarkersWidget;
class Connection;

class Server : public QTcpServer
{
    Q_OBJECT

public:
    explicit Server(QObject *parent, SharedObjects *sharedObjects, PlayersPage *playersPage);
    void close();

    void broadcastRegisterPlayers(const QString &name, const int marker, const int clientId = 0);
    void broadcastDeletePlayers(const QString &name, const int clientId = 0);
    void broadcastDeleteMarkedPlayers(const QString &markerName, int markerId,
                                      const int clientId = 0);
    void broadcastDeleteUnmarkedPlayers(const int clientId = 0);
    void broadcastDeleteAllPlayers(const int clientId = 0);
    void broadcastRenamePlayers(const QString &name, const QString &newName,
                                const int clientId = 0);
    void broadcastSetMarker(const QString &name, const QString &markerName, int markerId,
                            const int clientId = 0);
    void broadcastRemoveMarker(const QString &name, const int clientId = 0);
    void broadcastListSetMarker(const QString &markerName, const int markerId,
                                const int clientId = 0);
    void broadcastListRemoveMarker(const QString &markerName, const int markerId,
                                   const int clientId = 0);
    void broadcastListRemoveAllMarkers(const int clientId = 0);

    void broadcastAddMarker(const QString &name, const QColor &color,
                            const Marker::Sorting sorting, const int clientId = 0);
    void broadcastDeleteMarker(const QString &name, const int id, const int sequence,
                               const int clientId = 0);
    void broadcastMoveMarkerUp(const QString &name, const int id, const int sequence,
                               const int clientId = 0);
    void broadcastMoveMarkerDown(const QString &name, const int id, const int sequence,
                                 const int clientId = 0);
    void broadcastEditMarker(const QString &name, const int id, const QString &newName,
                             const QColor &color, const Marker::Sorting sorting,
                             const int clientId = 0);

    void broadcastFinishRegistration(const int clientId = 0);

    int clientCount() const;

signals:
    void protocolError(const QString &text);
    void message(const QString &text);
    void statusUpdate(const QString &text);
    void clientCountChanged(int count);
    void nameEdited();

protected:
    virtual void incomingConnection(qintptr descriptor) override;

private slots:
    void checkHandshakeTimeout(Connection *connection);
    void removeConnection(Connection *connection);
    void processHandshake(Connection *connection, const QString &phase);
    void processClientRequest(Connection *connection, const QString &action,
                              const QJsonObject &jsonObject);

private: // Functions
    void broadcastMessage(QJsonObject &jsonObject, const int clientId, const QString &messageText);
    const QString clientString(Connection *connection) const;

private: // Variables
    Database *m_db;
    PlayersPage *m_playersPage;
    MarkersWidget *m_markersWidget;
    QList<Connection *> m_pendingConnections;
    QList<Connection *> m_connections;
    int m_clientIdCounter = 0;
    const QString m_locale;

};

#endif // SERVER_H
