/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "MarkersTemplatePage.h"

#include "shared/Database.h"
#include "shared/Settings.h"
#include "shared/TableDelegate.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QLabel>
#include <QTableWidget>
#include <QPushButton>
#include <QHeaderView>
#include <QMessageBox>

MarkersTemplatePage::MarkersTemplatePage(QWidget *parent, Database *db, Settings *settings)
    : AbstractSettingsPage(parent, tr("Vorlage Markierungen")),
      m_db(db), m_settings(settings)
{
    QLabel *markersTemplateLabel = new QLabel(tr("Folgende Markierungen werden in neuen "
                                                 "Turnierdatenbanken automatisch angelegt:"));
    markersTemplateLabel->setWordWrap(true);
    layout()->addWidget(markersTemplateLabel);

    m_templateDisplay = new QTableWidget;
    m_templateDisplay->verticalHeader()->hide();
    m_templateDisplay->setShowGrid(false);
    m_templateDisplay->setSelectionMode(QTableWidget::NoSelection);
    m_templateDisplay->setFocusPolicy(Qt::NoFocus);
    m_templateDisplay->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_templateDisplay->setItemDelegate(new TableDelegate(m_templateDisplay));
    m_templateDisplay->setColumnCount(2);
    m_templateDisplay->setHorizontalHeaderLabels({ tr("Name/Farbe"), tr("Sortierung") });
    layout()->addWidget(m_templateDisplay);

    layout()->addStretch();

    QHBoxLayout *controlLayout = new QHBoxLayout;
    layout()->addLayout(controlLayout);
    controlLayout->addStretch();

    QPushButton *restoreTemplateButton = new QPushButton(tr("Zurücksetzen"));
    connect(restoreTemplateButton, &QPushButton::clicked,
            this, &MarkersTemplatePage::restoreTemplate);
    controlLayout->addWidget(restoreTemplateButton);

    QPushButton *adoptCurrentmarkersButton = new QPushButton(tr("Aktuelle Markierungen "
                                                                "übernehmen"));
    connect(adoptCurrentmarkersButton, &QPushButton::clicked,
            this, &MarkersTemplatePage::adoptCurrentmarkers);
    controlLayout->addWidget(adoptCurrentmarkersButton);
    adoptCurrentmarkersButton->setEnabled(! m_db->dbFile().isEmpty());

    m_template = m_settings->markersTemplate();
    updateTemplateDisplay();
}

void MarkersTemplatePage::updateTemplateDisplay()
{
    m_templateDisplay->setRowCount(m_template.count());
    for (int i = 0; i < m_template.count(); i++) {
        QTableWidgetItem *item = new QTableWidgetItem;
        m_templateDisplay->setItem(i, 0, item);
        item->setText(m_template.at(i).name.isEmpty() ? tr("(unmarkiert)") : m_template.at(i).name);
        item->setForeground(m_template.at(i).color);
        m_templateDisplay->setItem(i, 1, new QTableWidgetItem(
            m_template.at(i).sorting == Marker::Sorting::NewBlock ? tr("Neuer Block")
                                                                  : tr("Fortlaufend")));
    }
    m_templateDisplay->resizeColumnsToContents();
}

void MarkersTemplatePage::restoreTemplate()
{
    m_template = m_settings->defaultMarkersTemplate();
    updateTemplateDisplay();
}

void MarkersTemplatePage::adoptCurrentmarkers()
{
    m_template = m_db->markers();
    updateTemplateDisplay();
}

void MarkersTemplatePage::saveSettings()
{
    m_settings->saveMarkersTemplate(m_template);
}
