/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTDISCOVERENGINE_H
#define ABSTRACTDISCOVERENGINE_H

// Qt includes
#include <QObject>
#include <QHostAddress>
#include <QJsonObject>

// Qt classes
class QUdpSocket;

class AbstractDiscoverEngine : public QObject
{
    Q_OBJECT

public:
    explicit AbstractDiscoverEngine(QObject *parent);
    bool listen();
    QString errorString();
    void close();

signals:
    void invalidData(const QHostAddress &address);

protected: // Functions
    virtual void processData(const QJsonObject &jsonObject, const QHostAddress &senderAddress) = 0;
    void broadcastData(const QJsonObject &jsonObject);
    void sendData(const QJsonObject &jsonObject, const QHostAddress &address);

private slots:
    void readDatagram();

private: // Variables
    QUdpSocket *m_socket;

};

#endif // ABSTRACTDISCOVERENGINE_H
