/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTNETWORKENGINE_H
#define ABSTRACTNETWORKENGINE_H

// Qt includes
#include <QObject>
#include <QHostAddress>

// Qt classes
class QJsonObject;
class QTcpSocket;

class AbstractNetworkEngine : public QObject
{
    Q_OBJECT

public:
    explicit AbstractNetworkEngine(QObject *parent);
    void sendData(const QJsonObject &jsonObject);
    QString ip() const;

    static QString colorToString(const QColor &color);
    static QColor colorFromValue(const QJsonValue &value);

signals:
    void protocolError(const QString &text);

protected slots:
    void readData();

protected: // Functions
    QTcpSocket * socket() const;
    virtual void dataReceived(const QString &action, const QJsonObject &jsonObject) = 0;

private: // Variables
    QTcpSocket *m_socket;

};

#endif // ABSTRACTNETWORKENGINE_H
