/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHENGINE_H
#define SEARCHENGINE_H

// Qt includes
#include <QObject>
#include <QMap>

// Local classes
class StringTools;

class SearchEngine : public QObject
{
    Q_OBJECT

public:
    enum SearchType {
        Default,
        DrawnPairsScore
    };

    explicit SearchEngine(QObject *parent, StringTools *stringTools);
    bool checkMatch(const QString &text, SearchType searchType = SearchType::Default) const;
    void clearSearchCache(SearchType searchType = SearchType::Default);
    void updateSearchCache(const QString &name, SearchType searchType = SearchType::Default);
    void setSearchTerm(const QString &text);
    QMap<QString, QStringList> checkListForPossibleDuplicates(const QStringList &list);

signals:
    void progress(int percent);

private: // Functions
    QStringList searchVariants(const QString &text);
    void addAllVariants(const QString &searchTerm, QStringList &variantsList);
    QString replaceUmlauts(QString text) const;
    QString stripDiacritis(const QString &text) const;
    bool checkMatch(const QString &text, const QMap<QString, QStringList> &variantsList) const;
    void generateSearchVariants(QStringList &variants, const QString &text) const;

private: // Variables
    StringTools *m_stringTools;
    QMap<QString, QStringList> m_playersNamesVariants;
    QMap<QString, QStringList> m_drawnPairsNamesVariants;
    QStringList m_searchTermRegularExpressions;

};

#endif // SEARCHENGINE_H
