/* Copyright (c) 2012-2018 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "SearchWidget.h"
#include "SearchLineEdit.h"

// Qt includes
#include <QHBoxLayout>
#include <QLabel>
#include <QDebug>

SearchWidget::SearchWidget(QWidget *parent) : QWidget(parent)
{
    QHBoxLayout *layout = new QHBoxLayout(this);

    m_searchLabel = new QLabel(tr("Suche:"));
    layout->addWidget(m_searchLabel);

    m_search = new SearchLineEdit;
    m_search->setClearButtonEnabled(true);
    connect(m_search, &QLineEdit::textChanged, this, &SearchWidget::processInput);
    layout->addWidget(m_search);

    m_matches = new QLabel;
    m_matches->hide();
    layout->addWidget(m_matches);
}

void SearchWidget::updateMatches(int matches)
{
    if (! m_matches->isVisible()) {
        m_matches->show();
    }

    if (matches == 0) {
        m_matches->setText(tr("<span style=\"color:red\">Keine Treffer</span>"));
    } else {
        m_matches->setText(tr("%1 Treffer").arg(QString::number(matches)));
    }
}

void SearchWidget::processInput(const QString &text)
{
    emit searchChanged(text);
    if (text.isEmpty()) {
        m_matches->hide();
    }
}

void SearchWidget::clear()
{
    m_search->clear();
    m_savedSearch.clear();
}

void SearchWidget::saveSearch()
{
    if (! m_savedSearch.isEmpty()) {
        return;
    }

    m_savedSearch = m_search->text();
    m_search->clear();
}

void SearchWidget::restoreSearch()
{
    if (m_savedSearch.isEmpty()) {
        return;
    }

    m_search->setText(m_savedSearch);
    m_savedSearch.clear();
}
