/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SERVER_PROTOCOL_H
#define SERVER_PROTOCOL_H

#include <QJsonObject>
#include <QDataStream>

namespace ServerProtocol
{

enum RejectReason {
    OutdatedRequest,
    RegistrationFinished,
    FinishRegistrationNotPossible
};

const QDataStream::Version STREAM_VERSION = QDataStream::Qt_5_7;

constexpr int PROTOCOL_VERSION = 1;
const QLatin1String SERVER_SIGNATURE("muckturnier server");

const QLatin1String KEY_ACTION       ("action");
const QLatin1String KEY_CLIENT_ID    ("client id");
const QLatin1String KEY_CHECKSUM     ("checksum");
const QLatin1String KEY_REJECT_REASON("reject reason");

// Invalid requests
const QLatin1String ACTION_REJECTED("request rejected");
const QLatin1String ACTION_UNKNOWN ("unknown action");

// Actions
// =======

const QLatin1String ACTION_HANDSHAKE("handshake");

const QLatin1String ACTION_PLAYERS_REGISTER       ("register players");
const QLatin1String ACTION_PLAYERS_RENAME         ("rename players");
const QLatin1String ACTION_PLAYERS_DELETE         ("delete players");
const QLatin1String ACTION_PLAYERS_DELETE_MARKED  ("delete marked players");
const QLatin1String ACTION_PLAYERS_DELETE_UNMARKED("delete unmarked players");
const QLatin1String ACTION_PLAYERS_DELETE_ALL     ("delete all players");
const QLatin1String ACTION_PLAYERS_SET_MARKER     ("set players marker");
const QLatin1String ACTION_PLAYERS_REMOVE_MARKER  ("remove players marker");

const QLatin1String ACTION_PLAYERS_LIST_SET_MARKER        ("set players list marker");
const QLatin1String ACTION_PLAYERS_LIST_REMOVE_MARKER     ("remove players list marker");
const QLatin1String ACTION_PLAYERS_LIST_REMOVE_ALL_MARKERS("remove all players list markers");

const QLatin1String ACTION_MARKERS_ADD      ("add marker");
const QLatin1String ACTION_MARKERS_EDIT     ("edit marker");
const QLatin1String ACTION_MARKERS_DELETE   ("delete marker");
const QLatin1String ACTION_MARKERS_MOVE_UP  ("move marker up");
const QLatin1String ACTION_MARKERS_MOVE_DOWN("move marker down");

const QLatin1String ACTION_FINISH_REGISTRATION("finish registration");

const QLatin1String ACTION_RANKING_REQUEST("request ranking");

// Handshake
// =========

const QLatin1String HANDSHAKE_KEY_PHASE   ("phase");
const QLatin1String HANDSHAKE_PHASE_FAILED("failed");

// Welcome message
const QLatin1String HANDSHAKE_PHASE_INITIALIZATION("initialization");
const QLatin1String HANDSHAKE_KEY_SERVER_SIGNATURE("server signature");
const QLatin1String HANDSHAKE_KEY_PROTOCOL_VERSION("protocol version");
const QJsonObject HANDSHAKE_MESSAGE_INITIALIZE {
    { KEY_ACTION,                     ACTION_HANDSHAKE },
    { HANDSHAKE_KEY_PHASE,            HANDSHAKE_PHASE_INITIALIZATION },
    { HANDSHAKE_KEY_SERVER_SIGNATURE, SERVER_SIGNATURE },
    { HANDSHAKE_KEY_PROTOCOL_VERSION, PROTOCOL_VERSION }
};

// Synchronization request
const QLatin1String HANDSHAKE_PHASE_SYNCHRONIZE("synchronization");
const QJsonObject HANDSHAKE_MESSAGE_SYNCHRONIZE {
    { KEY_ACTION,          ACTION_HANDSHAKE },
    { HANDSHAKE_KEY_PHASE, HANDSHAKE_PHASE_SYNCHRONIZE }
};

// Synchronization answer
const QLatin1String HANDSHAKE_KEY_LOCALE            ("locale");
const QLatin1String HANDSHAKE_KEY_SETTINGS          ("settings");
const QLatin1String HANDSHAKE_KEY_TOURNAMENT_MODE   ("tournament mode");
const QLatin1String HANDSHAKE_FIXED_PAIRS           ("fixed pairs");
const QLatin1String HANDSHAKE_SINGLE_PLAYERS        ("single players");
const QLatin1String HANDSHAKE_KEY_BOOGER_SCORE      ("booger score");
const QLatin1String HANDSHAKE_KEY_BOOGERS_PER_ROUND ("boogers per round");
const QLatin1String HANDSHAKE_KEY_TOURNAMENT_STARTED("tournament started");
const QLatin1String HANDSHAKE_KEY_MARKERS           ("markers");
const QLatin1String HANDSHAKE_KEY_MARKER_ID         ("marker id");
const QLatin1String HANDSHAKE_KEY_MARKER_NAME       ("marker name");
const QLatin1String HANDSHAKE_KEY_MARKER_COLOR      ("color");
const QLatin1String HANDSHAKE_KEY_MARKER_SORTING    ("sorting");
const QLatin1String HANDSHAKE_KEY_PLAYERS           ("players");

// Synchronization success, request/allocate an id
const QLatin1String HANDSHAKE_PHASE_ALLOCATE_ID("id allocation");
const QJsonObject HANDSHAKE_MESSAGE_ALLOCATE_ID {
    { KEY_ACTION,          ACTION_HANDSHAKE },
    { HANDSHAKE_KEY_PHASE, HANDSHAKE_PHASE_ALLOCATE_ID }
};

// Players list changes
// ====================

const QLatin1String PLAYERS_KEY_NAME       ("name");
const QLatin1String PLAYERS_KEY_NEW_NAME   ("new name");
const QLatin1String PLAYERS_KEY_MARKER_ID  ("marker id");
const QLatin1String PLAYERS_KEY_MARKER_NAME("marker name");

// Marker changes
// ==============

const QLatin1String MARKERS_KEY_NAME    ("name");
const QLatin1String MARKERS_KEY_COLOR   ("color");
const QLatin1String MARKERS_KEY_SORTING ("sorting");
const QLatin1String MARKERS_KEY_ID      ("id");
const QLatin1String MARKERS_KEY_NEW_NAME("new name");
const QLatin1String MARKERS_KEY_SEQUENCE("sequence");

// Ranking comparison
// ==================

const QLatin1String RANKING_KEY_AVAILABLE     ("ranking available");
const QLatin1String RANKING_KEY_NAMES         ("names");
const QLatin1String RANKING_KEY_BOOGERS       ("boogers");
const QLatin1String RANKING_KEY_GOALS         ("goals");
const QLatin1String RANKING_KEY_OPPONENT_GOALS("opponent goals");
const QLatin1String RANKING_KEY_PLAYED_BOOGERS("played boogers");

}

#endif // SERVER_PROTOCOL_H
