/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SETTINGS_H
#define SETTINGS_H

// Local includes
#include "Marker.h"

// Qt includes
#include <QString>
#include <QSettings>

// Qt classes
class QAction;

class Settings : public QSettings
{
    Q_OBJECT

public:
    struct TournamentTemplate {
        bool pairMode;
        int boogerScore;
        int boogersPerRound;
        bool autoSelectPairs;
        bool noTableNumbers;
    };

    explicit Settings(QObject *parent);

    bool saveMainWindowGeometryEnabled() const;
    void enableSaveMainWindowGeometry(bool state);
    void saveMainWindowGeometry(const QByteArray &data);
    QByteArray mainWindowGeometry() const;

    bool savePlayersPageColumnsEnabled() const;
    void enableSavePlayersPageColumns(bool state);
    void savePlayersPageColumns(const QList<bool> &visibility);
    QList<bool> playersPageColumns() const;

    bool saveLastDbPathEnabled() const;
    void enableSaveLastDbPath(bool state);
    void saveLastDbPath(const QString &dbPath);
    QString lastDbPath() const;

    void saveTournamentTemplate(const TournamentTemplate &tournamentTemplate);
    TournamentTemplate defaultTournamentTemplate() const;
    TournamentTemplate tournamentTemplate() const;

    void saveDbFileNameTemplate(const QString &text);
    QString dbFileNameTemplate() const;

    void saveMarkersTemplate(const QList<Marker::Marker> &markers);
    QList<Marker::Marker> defaultMarkersTemplate();
    QList<Marker::Marker> markersTemplate();

    void saveBoogerSymbol(const QString &text);
    QString boogerSymbol() const;
    void saveNamesSeparator(const QString &text);
    QString namesSeparator() const;

private: // Functions
    bool parseMarkerData(int &id, const QJsonValue &value, QList<Marker::Marker> &markers);
    void appendMarker(const QString &name, const QColor &color, Marker::Sorting sorting,
                      int id, QList<Marker::Marker> &markers) const;

};

#endif // SETTINGS_H
