/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "SharedObjects.h"
#include "StringTools.h"
#include "SearchEngine.h"
#include "Database.h"
#include "ResourceFinder.h"
#include "Settings.h"

SharedObjects::SharedObjects(QObject *parent) : QObject(parent)
{
    m_stringTools = new StringTools(this);
    m_searchEngine = new SearchEngine(this, m_stringTools);
    m_database = new Database(this, m_stringTools);
    m_resourceFinder = new ResourceFinder(this, m_stringTools);
    m_settings = new Settings(this);

    m_database->setNamesSeparator(m_settings->namesSeparator());
    m_stringTools->setBoogerSymbol(m_settings->boogerSymbol());
}

StringTools * SharedObjects::stringTools() const
{
    return m_stringTools;
}

SearchEngine * SharedObjects::searchEngine() const
{
    return m_searchEngine;
}

Database * SharedObjects::database() const
{
    return m_database;
}

ResourceFinder * SharedObjects::resourceFinder() const
{
    return m_resourceFinder;
}

Settings * SharedObjects::settings() const
{
    return m_settings;
}
