/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STRINGTOOLS_H
#define STRINGTOOLS_H

// Qt includes
#include <QObject>
#include <QLocale>
#include <QCollator>

class StringTools : public QObject
{
    Q_OBJECT

public:
    explicit StringTools(QObject *parent);
    void sortList(QStringList &list) const;
    QString toLower(const QString &text) const;
    void appendIfNotExists(QStringList &list, const QString &text) const;
    void appendIfNotExists(QList<int> &list, const int value) const;
    void removeIntersection(QStringList &list1, QStringList &list2) const;
    int levenshteinDistance(const QString &source, const QString &target) const;
    QMap<int, int> combinedLevenshteinDistance(const QString &text,
                                               const QStringList &candidates) const;
    QMap<int, int> matchLists(QStringList &source, QStringList &target) const;
    QString longestCommonSubsequence(const QString &source, const QString &target) const;
    QStringList diffList(QString text, QString longestCommonSubsequence) const;
    QString boogify(int score, int boogerScore) const;
    QString locale() const;
    QString defaultLocale() const;
    void setLocale(const QString &name);
    void resetLocale();
    bool localeChanged() const;
    void setBoogerSymbol(const QString &text);
    const QString & boogerSymbol() const;

private: // Variables
    QLocale m_locale;
    QCollator m_collator;
    bool m_localeChanged = false;
    QString m_boogerSymbol;

};

#endif // STRINGTOOLS_H
