/* Copyright (c) 2012-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "TableDelegate.h"

// Qt includes
#include <QPainter>
#include <QBrush>

TableDelegate::TableDelegate(QObject *parent, HighlightMode highlightMode)
    : QStyledItemDelegate(parent), m_highlightMode(highlightMode)
{
    m_topLineColor = QColor(170, 170, 170);
    m_highlightBackground = QPixmap(1, 1);
    m_highlightBackground.fill(QColor(240, 240, 240));
}

void TableDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                          const QModelIndex &index) const
{
    painter->save();

    bool drawBackground = false;
    bool drawTopLine = false;

    if ((m_highlightMode == HighlightMode::AllRows && index.row() % 2 != 0)
        || (m_highlightMode == HighlightMode::EvenRows && index.row() / 2 % 2 != 0)) {

        drawBackground = true;
    }

    if (m_highlightMode == HighlightMode::Custom) {
        drawBackground = index.data(Display::DrawBackground).isValid();
    }

    if (drawBackground) {
        QBrush brush = option.backgroundBrush;
        brush.setTexture(m_highlightBackground);
        painter->fillRect(option.rect, brush);
    }

    if (m_highlightMode != HighlightMode::Custom
        && index.row() != 0
        && (m_highlightMode == HighlightMode::AllRows || index.row() % 2 == 0)) {

        drawTopLine = true;
    }

    if (m_highlightMode == HighlightMode::Custom) {
        drawTopLine = index.data(Display::DrawTopLine).isValid();
    }

    if (drawTopLine) {
        painter->setPen(m_topLineColor);
        painter->drawLine(option.rect.topLeft(), option.rect.topRight());
    }

    painter->restore();

    drawText(painter, option, index);
}

void TableDelegate::drawText(QPainter *painter, const QStyleOptionViewItem &option,
                             const QModelIndex &index) const
{
    QStyledItemDelegate::paint(painter, option, index);
}
