/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TITLEDIALOG_H
#define TITLEDIALOG_H

// Qt includes
#include <QDialog>
#include <QDialogButtonBox>
#include <QPushButton>

// Qt classes
class QVBoxLayout;
class QLabel;

class TitleDialog : public QDialog
{
    Q_OBJECT

public:
    enum DeleteMode {
        DeleteOnClose,
        NoDelete
    };

    void setTitle(const QString &title);
    void resizeToContents();

    void addButtonBox();
    void addButtonBox(QDialogButtonBox::StandardButtons buttons);
    void setOkButtonText(const QString &text);
    bool okButtonEnabled() const;
    void setButtonText(QDialogButtonBox::StandardButton button, const QString &text);

public slots:
    void setOkButtonEnabled(bool state);

signals:
    void updateStatus(const QString &text);

protected:
    explicit TitleDialog(QWidget *parent, DeleteMode deleteMode = DeleteMode::DeleteOnClose);
    QVBoxLayout * mainLayout() const;
    QVBoxLayout * layout() const;
    QDialogButtonBox * buttonBox() const;

protected slots:
    virtual void accept();

private: // Variables
    QVBoxLayout *m_mainLayout;
    QVBoxLayout *m_layout;
    QLabel *m_title;
    QDialogButtonBox *m_buttonBox;

};

#endif // TITLEDIALOG_H
